/* File: code03e8.c
   ============================
 This file contains a patch for the trap "FrmDrawForm".  All it does it pass
control over to the standard FrmDrawForm function and then and time after the form title.

   Based on the GreenLightHack by Douglas I. Anderson (dianders@cs.stanford.edu)
   special thank to him.
   
   renaud Malaval (renaud.malaval2@fnac.net)

	Hack patch info
   ---------------
   code id:   0x1000
   atrap:	  0xA171 (sysTrapFrmDrawForm) 41329

   History
   -------
   15/06/00 Created
*/

// ----------------------------------
// Interfaces...
// ----------------------------------

/* System-wide */
#include <PalmTypes.h>
#include <SoundMgr.h>
#include <FeatureMgr.h>

#define TRACE_OUTPUT		TRACE_OUTPUT_OFF

/* Hack specific */
#include "ResourceDefines.h"
#include "ClockEveryWhere.h"
#include "utils.h"

// ----------------------------------
// Local types...
// ----------------------------------

typedef void (*FrmDrawFormFn) (FormPtr frmP);

// ----------------------------------
// Local function prototypes...
// ----------------------------------

// ----------------------------------
// Trap Handler...
// ----------------------------------

/* Function: MyFrmDrawForm()
   --------------------------
   This is the entry function of this patch.  All system calls to FrmDrawForm
   will be redirected here.  Basically, if the caller is actually using this
   function to display the form.
   
   @param frm      Tells the form to patch
   @return         nothing.
 */

// This is the code that will be called whenever ANY app calls FrmDrawForm,
// and CLockEveryWhere is enabled in HackMaster. HackMaster is not smart about the
// content of this file; it will simply jump to the first executable statement
// in this file. If you need helper functions, prototype them above, and
// define them AFTER the trap patch.
void	MyFrmDrawForm (FormPtr frmP)
{
	FrmDrawFormFn	theOldTrap;
	UInt32 			theTemp;
	CEWpref_t		thePrefs;
	
    // get the address of the old FrmDrawForm() from HackMaster
	if (FtrGet (kCreatorCode, kHackMasterTrapID_1, &theTemp))
	{
		SndPlaySystemSound (sndWarning);
	}
	else
	{
		// set the function pointer to the old trap
		theOldTrap = (FrmDrawFormFn)theTemp;
		// Since this hack is patching FrmDrawForm, there's not really much
		// useful work we can do BEFORE making the call to the OS, to get the
		// event, so we call the linked FrmDrawForm() first. This is not always
		// the case; sometimes you want to do something, then call the original
		// trap routine, then maybe do something else. This design decision is far
		// beyond the scope of this Sample.
		theOldTrap (frmP);

		Utils_GetHackPref( &thePrefs);

		Utils_DrawTheClock( frmP, true, &thePrefs);
	}
}	// MyFrmDrawForm




